/* 
 * keyshades
 * a tool for disabling the treo keyboard backlight
 * copyright (c) 2005, daniel grobe sachs <dgsachs@nekito.net>
 *
 * This is free software. See included LICENSE file for details.
 */

#include <PalmOS.h>
#include <palmOne_68K.h>
#include <68K/System/HsExt.h>
#include <Common/System/HsExtCommon.h>

#include "resource.h"

#define STARTUP_CODE_SIZE	152		// size of prc-tools crt0.c

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags);

// Endian swapper
#define EndianSwap16(n) (((((UInt16) n) << 8) & 0xFF00) | \
		         ((((UInt16) n) >> 8) & 0x00FF))

static void RegisterForNotifications();
static void DeregisterNotifications();

#define FTR_PTR 	1
#define POPUPTIME_FTR	100
#define POPUP_CLEAR_TIME 15 // seconds

#define CAUSERIE_CHAT_WINDOW 3571
#define CAUSERIE_HIDE_CONTROL 3582

// Notification handler
static Err HandleNotification(SysNotifyParamType *np)
{
        UInt32 time;
	UInt32 clear_time = 0;
	UInt16 frm;
        struct EventType newevent;
        EventType *eventP = (EventType*)np->notifyDetailsP;

        // Check if we're deleting the application
        if( np->notifyType == sysNotifyDeleteProtectedEvent )
        {
                if( ((SysNotifyDBInfoType *)np->notifyDetailsP)->creator == CREATOR )
                {
                        DeregisterNotifications();
                        np->handled = true;
                }

                return 0;
        }

        // Is this a queue event?
        if( np->notifyType != sysNotifyEventDequeuedEvent )
                return 0;

	frm = FrmGetActiveFormID();
	FtrGet(CREATOR,POPUPTIME_FTR,&clear_time);
	time = TimGetTicks();

	// Open Causerie window? 
/*
	if( ((eventP->eType == EndianSwap16(frmOpenEvent)) || 
	     (eventP->eType == EndianSwap16(frmGotoEvent))) && 
	    (eventP->data.frmOpen.formID == EndianSwap16(CAUSERIE_CHAT_WINDOW)) )
	{
                //SndPlaySystemSound(sndInfo);
		FtrSet(CREATOR,POPUPTIME_FTR,time+(POPUP_CLEAR_TIME * SysTicksPerSecond()));
	} */

	if( frm == CAUSERIE_CHAT_WINDOW )
	{
		if( !clear_time )
			FtrSet(CREATOR,POPUPTIME_FTR,
				time+(POPUP_CLEAR_TIME * SysTicksPerSecond()));
	}
			
	// User activity (temporarily) overrides clear
	if( (frm != CAUSERIE_CHAT_WINDOW) || 
	    (eventP->eType == EndianSwap16(penDownEvent)) ||
	    (eventP->eType == EndianSwap16(penMoveEvent)) ||
	    (eventP->eType == EndianSwap16(penUpEvent))   ||
	    (eventP->eType == EndianSwap16(keyDownEvent)) )
	{
		FtrSet(CREATOR,POPUPTIME_FTR,0);
		clear_time = 0;
	}

	else if( clear_time && (time > clear_time) )
	{
		char *buf[128];
		EventType *newEvent;
		MemSet(buf,128,0);

                //SndPlaySystemSound(sndInfo);

		newEvent = (EventType *)buf;
		newEvent->eType = ctlSelectEvent;
		newEvent->data.ctlSelect.controlID = CAUSERIE_HIDE_CONTROL;
		EvtAddEventToQueue(newEvent);
		
		FtrSet(CREATOR,POPUPTIME_FTR,0);
	}
}

static void RegisterForNotifications()
{
	MemHandle hndl;
	void *memptr;
	void *ftrptr;
	void *hn_ptr;

	UInt16 cardNo;
	LocalID dbID;

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	hndl = DmGet1Resource('code',1);
	memptr = MemHandleLock(hndl);

	FtrPtrNew(CREATOR, FTR_PTR, MemHandleSize(hndl),&ftrptr);
	DmWrite(ftrptr, 0, memptr, MemHandleSize(hndl));

	MemHandleUnlock(hndl);
	DmDatabaseProtect(cardNo, dbID, true);

	hn_ptr = ((void *)ftrptr)+STARTUP_CODE_SIZE;

	SysNotifyRegister(cardNo, dbID, sysNotifyEventDequeuedEvent, hn_ptr, 
			0, NULL);

 	SysNotifyRegister(cardNo, dbID, sysNotifyDeleteProtectedEvent, NULL,
			0, NULL);
}

static void DeregisterNotifications()
{	
	Int32 val;
	void *ftrptr;

	UInt16 cardNo;
	LocalID dbID;

	val = 0; FtrGet(CREATOR, FTR_PTR, &val); ftrptr = (void *)val;

	// Check if we're registered
	if( ftrptr == NULL )
		return; 

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent,
			0);

 	SysNotifyUnregister(cardNo, dbID, sysNotifyDeleteProtectedEvent,
			0);

	// release feature memory
	FtrPtrFree(CREATOR, FTR_PTR);
	DmDatabaseProtect(cardNo, dbID, false);
}


//// PilotMain /////////////////////////////////////////////////////////
// It all starts here...

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) 
{
	Err err = 0;
	EventType event;

	switch( launchCode ) 
	{
		case sysAppLaunchCmdSystemReset :
			RegisterForNotifications();
			break;

		case sysAppLaunchCmdNotify : 
			HandleNotification((SysNotifyParamType *)cmdPBP);
			break;

		case sysAppLaunchCmdNormalLaunch :
			// Well, uh, we don't have a UI, so we just quit
			MemSet( &event, sizeof(event), 0);

			event.eType = appStopEvent;
			EvtAddEventToQueue(&event);
			break;

		default:
			break;
	}


	return err;
}

